
#pragma once

#include <Base_DAQ.h>
#include <Base_Configurator.h>
#include <thread>

namespace inaf::oasbo::DAQ {

/**
 * @brief The AstriDAQ class represents the ASTRI Data Acquisition (DAQ) system.
 * 
 * This class inherits from the BaseDAQ class and provides additional functionality specific to the ASTRI DAQ system.
 * It handles the connection to the provider, packet reception and processing, and state management.
 */
class AstriDAQ: public BaseDAQ {
protected:

    /**
     * @brief The scheduling_block struct represents a scheduling block for data acquisition.
     * 
     * It contains information such as the date, target ID, observation mode, telescope ID, and run ID.
     */
	struct scheduling_block {
		std::string date, target_id, obs_mode;
		int tel_id, run_id;
	};

	bool firstIdle;
    int daq_id = 0;
    void updateDaqIDinConf(); // update daq_id in the configuration in order to keep track of the last daq_id used.

	std::thread checkProvConn; // a thread to check, in loop, if the connection to the provider is still up.
    int connectProvider(); // the function used in checkProvConn thread.
	bool checkProvThreadFlag = false; // the flag to stop the checkProvConn thread.

	std::thread receiveAndDeliverThread; // a thread to receive and deliver packets.
	bool runningFlag = false; // a flag that tells if the DAQ should be run.
	int receiveAndDeliverPacket(); // receive packer from the receiver, process, and deliver to the archiver, provider etc.

    /**
     * @brief Prints the given message to the given output stream.
     * 
     * @param os The output stream to print to.
     * @param message The message to print.
     */
	void printLog(std::ostream &os, std::string message);

public:
    /**
     * @brief Constructs an AstriDAQ object.
     */
    AstriDAQ();

    /**
     * @brief Destroys the AstriDAQ object.
     */
    ~AstriDAQ();

	std::string getStateStr(Status) override;

    void start() override;

 
    void stop() override;


    void switchState(const Status) override;

    int deliverPacket() override;

    /**
     * @brief Updates the observers of the data acquisition system.
     */
    void updateObservers();

    /**
     * @brief Updates the archiver observers of the data acquisition system.
     */
    void updateArchiverObservers();

    /**
     * @brief Sets the daqID
     * @param id The daqID to set.
     */
    void setDaqID(int id);

    /**
     * @brief Gets the daqID.
     * 
     * @return The daqID.
     */
    int getDaqID();

    /**
     * @brief Increments the daqID.
    */
    void incrementDaqID();

    /**
     * @brief Computes the new destination based on the given status. // the file name based on the Astri Ma file naming convention.
     * 
     * @param status The status to compute the new destination for.
     * @return The computed new destination.
     */
    std::string computeNewDestination(Status status);

    scheduling_block scheduling_block;

    friend class AstriDAQBuilder;

};

/**
 * @brief The AstriDAQBuilder class is responsible for building an AstriDAQ object.
 */
class AstriDAQBuilder {
  protected:
      AstriDAQ* daq;

  public:
      /**
       * @brief Constructs an AstriDAQBuilder object.
       */
      AstriDAQBuilder();

      /**
       * @brief Destroys the AstriDAQBuilder object.
       */
      ~AstriDAQBuilder();

      /**
       * @brief Resets the builder to its initial state.
       */
      void reset();

      /**
       * @brief Configures the builder from the given BaseConfigurator object.
       * 
       * @param conf The BaseConfigurator object to configure from.
       * @return The configured AstriDAQBuilder object.
       */
      AstriDAQBuilder* configFrom(Configurators::BaseConfigurator &conf);

      /**
       * @brief Sets the protocol for the data acquisition system.
       * 
       * @param receiver The BaseReceiver object representing the protocol.
       * @return The configured AstriDAQBuilder object.
       */
      AstriDAQBuilder* setProtocol(Receivers::BaseReceiver* receiver);

      /**
       * @brief Sets the archiver for the data acquisition system.
       * 
       * @param archiver The BaseArchiver object representing the archiver.
       * @return The configured AstriDAQBuilder object.
       */
      AstriDAQBuilder* setArchiver(Archivers::BaseArchiver* archiver);

      /**
       * @brief Sets the provider for the data acquisition system.
       * 
       * @param provider The BaseProvider object representing the provider.
       * @return The configured AstriDAQBuilder object.
       */
      AstriDAQBuilder* setProvider(Providers::BaseProvider* provider);

      /**
       * @brief Sets the packet monitor for the data acquisition system.
       * 
       * @param monitor The BasePacketMonitor object representing the packet monitor.
       * @return The configured AstriDAQBuilder object.
       */
      AstriDAQBuilder* setMonitor(PacketMonitors::BasePacketMonitor* monitor);

      /**
       * @brief Sets the packet for the data acquisition system.
       * 
       * @param packet The BasePacket object representing the packet.
       * @return The configured AstriDAQBuilder object.
       */
      AstriDAQBuilder* setPacket(Packets::BasePacket* packet);

      /**
       * @brief Sets the date ID for the scheduling block.
       * 
       * @param dateId The date ID to set.
       * @return The configured AstriDAQBuilder object.
       */
      AstriDAQBuilder* setDateId(std::string dateId);

      /**
       * @brief Sets the target ID for the scheduling block.
       * 
       * @param targetId The target ID to set.
       * @return The configured AstriDAQBuilder object.
       */
      AstriDAQBuilder* setTargetId(std::string targetId);

      /**
       * @brief Sets the observation mode for the scheduling block.
       * 
       * @param obsMode The observation mode to set.
       * @return The configured AstriDAQBuilder object.
       */
      AstriDAQBuilder* setObsMode(std::string obsMode);

      /**
       * @brief Sets the telescope ID for the scheduling block.
       * 
       * @param telId The telescope ID to set.
       * @return The configured AstriDAQBuilder object.
       */
      AstriDAQBuilder* setTelId(int telId);

      /**
       * @brief Sets the run ID for the scheduling block.
       * 
       * @param runId The run ID to set.
       * @return The configured AstriDAQBuilder object.
       */
      AstriDAQBuilder* setRunId(int runId);

      /**
       * @brief Sets the ID for the data acquisition system.
       * 
       * @param daqId The ID to set.
       * @return The configured AstriDAQBuilder object.
       */
      AstriDAQBuilder* setDaqId(int daqId);

      /**
       * @brief Gets the built AstriDAQ object.
       * 
       * @return The built AstriDAQ object.
       */
      AstriDAQ* getDAQ();

      static const std::string config_target;

      static const std::string receiver_key;
      static const std::string archiver_key;
      static const std::string provider_key;
      static const std::string packet_key;
      static const std::string monitor_key;
      static const std::string daq_id_key;

      static const std::string tcp_prot_key;
      static const std::string udp_prot_key;
      static const std::string file_rcv_key;
      static const std::string file_arch_key;
      static const std::string kafka_prov_key;
      static const std::string kafkaavro_prov_key;
      static const std::string redis_prov_key;
      static const std::string astri_ma_packet_key;
      static const std::string astri_horn_packet_key;
      static const std::string astri_simple_monitor_key;

      // Scheduling block params
      static const std::string sched_block_config_target;

      static const std::string date_id_key;
      static const std::string target_id_key;
      static const std::string obs_mode_key;
      static const std::string tel_id_key;
      static const std::string run_id_key;
  };

}
