
/**
 * @brief The AstriMySQLDAQObserver class is a derived class of BaseDAQ_Observer.
 * It provides functionality to observe and interact with a MySQL database for the ASTRI DAQ system.
 * Check Base_DAQ_Observer.h for more information.
 * 
 * In particular, this class is used to update the MySQL database with only the Archiver statistics of the ASTRI DAQ system (acquired files table).
 */
#pragma once

#include <Base_DAQ_Observer.h>
#include <Astri_DAQ.h>

#include <mysql-cppconn/mysql/jdbc.h>

#include <atomic>
#include <thread>
#include <mutex>
#include <condition_variable>

namespace inaf::oasbo::DAQ_observers {
class AstriMySQLDAQObserver: public BaseDAQ_Observer {
protected:
	sql::mysql::MySQL_Driver *driver = nullptr; // Pointer to the MySQL driver
	sql::Connection *con = nullptr; // Pointer to the MySQL connection
	std::string ip { }, username { }, password { }, dbname { }; // IP address, username, password, and database name of the MySQL server
	int port; // Port number of the MySQL server
	inaf::oasbo::DAQ::AstriDAQ *astri_daq; // Pointer to the ASTRI DAQ object
	sql::PreparedStatement *stmt = nullptr; // 		Pointer to the MySQL prepared statement
	bool file_is_open; // Flag to indicate if the file being archived is still open.

// names of the tables and fields in the database
	static const std::string FILES_TABLE;
	static const std::string CLOSED_FIELD;
	static const std::string CREATED_FIELD;
	static const std::string PATH_FIELD;
	static const std::string FILENAME_FIELD;
	static const std::string ID_FIELD;
	static const std::string DAQID_FIELD;
	static const std::string SCHED_BLOCK_FIELD;
	static const std::string SCHED_BLOCK_TABLE;
	static const std::string ACTIVE_FIELD;

public:
	/**
	 * @brief Constructs an AstriMySQLDAQObserver object.
	 * @param dataAcquisition The reference to the BaseDAQ object for data acquisition.
	 * @param ip The IP address of the MySQL database server.
	 * @param port The port number of the MySQL database server.
	 * @param username The username for connecting to the MySQL database server.
	 * @param password The password for connecting to the MySQL database server.
	 * @param dbname The name of the MySQL database.
	 */
	AstriMySQLDAQObserver(inaf::oasbo::DAQ::BaseDAQ &dataAcquisition, std::string ip, int port, std::string username, std::string password, std::string dbname);

	/**
	 * @brief Destroys the AstriMySQLDAQObserver object.
	 */
	~AstriMySQLDAQObserver();

	/**
	 * @brief Updates the packet statistics in the MySQL database.
	 */
	void updatePacketStats() override;

	/**
	 * @brief Updates the archiver statistics in the MySQL database.
	 */
	void updateArchiverStats() override;

	/**
	 * @brief Updates the provider statistics in the MySQL database.
	 */
	void updateProviderStats() override;

	/**
	 * @brief Updates the receiver statistics in the MySQL database.
	 */
	void updateReceiverStats() override;

	/**
	 * @brief Updates all statistics in the MySQL database.
	 */
	void updateAll() override;

	/**
	 * @brief Starts observing the MySQL database.
	 */
	void start() override;

	/**
	 * @brief Stops observing the MySQL database.
	 */
	void stop() override;
};
}
