
/**
 * @brief The AstriRedisDAQObserver class is a derived class of BaseDAQ_Observer that provides functionality for observing the ASTRI DAQ system using Redis.
 * 
 * This class establishes a connection with a Redis server and periodically updates the statistics of the ASTRI DAQ system.
 * It inherits the update methods from the BaseDAQ_Observer class and implements them to update the specific statistics related to packet, archiver, provider, and receiver.
 * The class also provides methods to start and stop the observation process.
 */
#pragma once

#include <Astri_DAQ.h>
#include <Base_DAQ_Observer.h>
#include <hiredis/hiredis.h>

#include <atomic>
#include <thread>
#include <mutex>
#include <condition_variable>
#include <set>

namespace inaf::oasbo::DAQ_observers {
class AstriRedisDAQObserver: public BaseDAQ_Observer {
protected:
	inaf::oasbo::DAQ::AstriDAQ *astri_daq; // Pointer to the ASTRI DAQ object
	redisContext *context = nullptr; // Pointer to the Redis context
	std::string ip{}; // IP address of the Redis server
	int port; // Port number of the Redis server

	bool checkConnThreadFlag = false; // Flag to indicate if the connection check thread is running
	std::thread checkConn; // Thread for checking the connection status
	bool isConnected(); // Method to check if the connection to the Redis server is established
	int connectLoop(); // Method to continuously attempt to establish a connection to the Redis server

    std::set<std::string> keys; // Set of keys used for updating the statistics
	bool stopFlag = false; // Flag to indicate if the observation process should stop
	std::mutex writeMutex; // Mutex for thread-safe writing
	std::thread observerThread; // Thread for updating the statistics
	std::condition_variable sleep_cv; // Condition variable for wake-up in observerThread.
	int updateInterval; // Interval seconds between updates
	void updateThread(); // Method for updating the statistics in observerThread.
	void updateDAQStats(); // Method for updating the ASTRI DAQ statistics

	static std::string lastopenfile_key; // Key for the last open file statistic
	static std::string lastclosedfile_key; // Key for the last closed file statistic
	static std::string mode_key; // Key for the mode statistic
	static std::string receiverSource_key; // Key for the receiver source statistic
	static std::string receiverConn_key; // Key for the receiver connection statistic
	static std::string provider_destination_key; // Key for the provider destination statistic

public:
	/**
	 * @brief Constructs an AstriRedisDAQObserver object.
	 * 
	 * @param dataAcquisition The reference to the BaseDAQ object representing the ASTRI DAQ system.
	 * @param ip The IP address of the Redis server.
	 * @param port The port number of the Redis server.
	 */
	AstriRedisDAQObserver(inaf::oasbo::DAQ::BaseDAQ &dataAcquisition,std::string ip, int port);

	/**
	 * @brief Destroys the AstriRedisDAQObserver object.
	 */
	~AstriRedisDAQObserver();

	void updatePacketStats() override; // Overrides the updatePacketStats method from the BaseDAQ_Observer class
	void updateArchiverStats() override; // Overrides the updateArchiverStats method from the BaseDAQ_Observer class
	void updateProviderStats() override; // Overrides the updateProviderStats method from the BaseDAQ_Observer class
	void updateReceiverStats() override; // Overrides the updateReceiverStats method from the BaseDAQ_Observer class
	void updateAll() override; // Overrides the updateAll method from the BaseDAQ_Observer class
	void start() override; // Overrides the start method from the BaseDAQ_Observer class
	void stop() override; // Overrides the stop method from the BaseDAQ_Observer class
};
}
