
#include <Astri_MySQL_DAQ_Observer.h>
#include <unistd.h>
#include <chrono>
#include <sstream>

#include <ctime>
#include <iomanip>

using namespace inaf::oasbo::DAQ_observers;

const std::string AstriMySQLDAQObserver::FILES_TABLE = "acquired_files";
const std::string AstriMySQLDAQObserver::CLOSED_FIELD = "closed_at";
const std::string AstriMySQLDAQObserver::CREATED_FIELD = "created_at";
const std::string AstriMySQLDAQObserver::PATH_FIELD = "folder";
const std::string AstriMySQLDAQObserver::FILENAME_FIELD = "name";
const std::string AstriMySQLDAQObserver::ID_FIELD = "id";
const std::string AstriMySQLDAQObserver::DAQID_FIELD = "daq_id";
const std::string AstriMySQLDAQObserver::SCHED_BLOCK_FIELD =
		"scheduling_block_id";

const std::string AstriMySQLDAQObserver::SCHED_BLOCK_TABLE =
		"schedulingblock_config";
const std::string AstriMySQLDAQObserver::ACTIVE_FIELD = "active";

AstriMySQLDAQObserver::AstriMySQLDAQObserver(inaf::oasbo::DAQ::BaseDAQ &daq,
		std::string ip, int port, std::string user, std::string password,
		std::string dbname) :
		inaf::oasbo::DAQ_observers::BaseDAQ_Observer(daq) {
	astri_daq = static_cast<inaf::oasbo::DAQ::AstriDAQ*>(&daq);
	this->ip = ip;
	this->port = port;
	try {
		driver = sql::mysql::get_mysql_driver_instance();
		con = driver->connect("tcp://" + ip + ":" + std::to_string(port), user,
				password);
		con->setSchema(dbname);
	} catch (sql::SQLException &e) {
		time_t now = time(nullptr);
		std::cerr << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
				<< "]\t[MYSQL Observer]\t" << e.what() << std::endl;
	}
	this->file_is_open = false;
}

AstriMySQLDAQObserver::~AstriMySQLDAQObserver() {
	if (stmt != nullptr)
		delete stmt;
	if (con != nullptr) {
		con->close();
		delete con;
	}
}

void AstriMySQLDAQObserver::updatePacketStats() {
}

void AstriMySQLDAQObserver::updateArchiverStats() {
	if (con == nullptr)
		return;
	// compute file_name and folder name
	std::string dest(dataAcquisition->getArchiverPtr()->getDest());
	size_t lastSlashPos = dest.find_last_of('/');
	std::string path, name;
	if (lastSlashPos != std::string::npos) {
		path = dest.substr(0, lastSlashPos);
		name = dest.substr(lastSlashPos + 1);
	} else {
		path = "";
		name = dest;
	}

	if (this->dataAcquisition->getArchiverPtr()->is_open() && !file_is_open) { // new file just opened, insert into the database with current open timestamp.
		std::stringstream query;
		try {
			std::string sched_block_id = "NULL";
			query << "SELECT MAX(" << ID_FIELD << ") FROM "
					<< SCHED_BLOCK_TABLE;
			query << " WHERE " << ACTIVE_FIELD << ";";
			stmt = con->prepareStatement(query.str());
			sql::ResultSet *res = stmt->executeQuery();
			if (res->next()) {
				int sbii = res->getInt(1);
				sched_block_id = std::to_string(sbii);
				delete res;
			}

			query.str("");
			query << "INSERT INTO " << FILES_TABLE;
			query << " SET " << CREATED_FIELD << " = NOW(),";
			query << PATH_FIELD << " = '" << path << "',";
			query << FILENAME_FIELD << " = '" << name << "',";
			query << DAQID_FIELD << " = " << astri_daq->getDaqID() << ",";
			query << SCHED_BLOCK_FIELD << " = " << sched_block_id << ";";

			stmt = con->prepareStatement(query.str());
			stmt->execute();
			file_is_open = true;
		} catch (sql::SQLException &e) {
			time_t now = time(nullptr);
			std::cerr << "["
					<< std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
					<< "]\t[MYSQL Observer]\t" << e.what() << " during query: "
					<< query.str() << std::endl;
		}
	} else { // the file has been closed, update the database with the current close timestamp.
		std::stringstream query;
		try {
			query << "UPDATE " << FILES_TABLE;
			query << " SET " << CLOSED_FIELD;
			query << " = NOW() ";
			query << " WHERE " << ID_FIELD
					<< " = ( SELECT id FROM ( SELECT MAX(" << ID_FIELD
					<< ") AS id FROM " << FILES_TABLE;
			query << " WHERE " << CLOSED_FIELD << " IS NULL";
			query << " AND " << PATH_FIELD << " = '" << path << "'";
			query << " AND " << FILENAME_FIELD << " = '" << name
					<< "') AS subquery );";

			stmt = con->prepareStatement(query.str());
			stmt->execute();

			file_is_open = false;
		} catch (sql::SQLException &e) {
			time_t now = time(nullptr);
			std::cerr << "["
					<< std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
					<< "]\t[MYSQL Observer]\t" << e.what() << " during query: "
					<< query.str() << std::endl;
		}
	}
}

void AstriMySQLDAQObserver::updateProviderStats() {

}

void AstriMySQLDAQObserver::updateReceiverStats() {
}

void AstriMySQLDAQObserver::updateAll() {
	if (con == nullptr)
		return;
	updateArchiverStats();
	updatePacketStats();
	updateProviderStats();
	updateReceiverStats();
}

void AstriMySQLDAQObserver::start() {

}

void AstriMySQLDAQObserver::stop() {

}
